@@ -11,18 +11,18 @@ import importlib
 import textwrap
 import signal
 import json
+import pygments
 from pygments import highlight
 from pygments.style import Style
 from pygments.token import (Name, Number, String, Keyword)
 from pygments.lexers import JsonLexer
 from pygments.formatters import Terminal256Formatter
-import jc.utils
 import jc.appdirs as appdirs
 
 
 class info():
-    version = '1.11.3'
-    description = 'jc cli output JSON conversion tool'
+    version = '1.13.2'
+    description = 'JSON CLI output utility'
     author = 'Kelly Brazil'
     author_email = 'kellyjonbrazil@gmail.com'
 
@@ -37,6 +37,7 @@ parsers = [
     'crontab',
     'crontab-u',
     'csv',
+    'date',
     'df',
     'dig',
     'dmidecode',
@@ -54,6 +55,7 @@ parsers = [
     'ini',
     'iptables',
     'jobs',
+    'kv',
     'last',
     'ls',
     'lsblk',
@@ -63,6 +65,7 @@ parsers = [
     'netstat',
     'ntpq',
     'passwd',
+    'ping',
     'pip-list',
     'pip-show',
     'ps',
@@ -70,11 +73,14 @@ parsers = [
     'shadow',
     'ss',
     'stat',
+    'sysctl',
     'systemctl',
     'systemctl-lj',
     'systemctl-ls',
     'systemctl-luf',
     'timedatectl',
+    'tracepath',
+    'traceroute',
     'uname',
     'uptime',
     'w',
@@ -86,8 +92,8 @@ parsers = [
 # List of custom or override parsers.
 # Allow any <user_data_dir>/jc/jcparsers/*.py
 local_parsers = []
-data_dir = appdirs.user_data_dir("jc", "jc")
-local_parsers_dir = os.path.join(data_dir, "jcparsers")
+data_dir = appdirs.user_data_dir('jc', 'jc')
+local_parsers_dir = os.path.join(data_dir, 'jcparsers')
 if os.path.isdir(local_parsers_dir):
     sys.path.append(data_dir)
     for name in os.listdir(local_parsers_dir):
@@ -98,8 +104,52 @@ if os.path.isdir(local_parsers_dir):
                 parsers.append(plugin_name)
 
 
-def set_env_colors():
+# We only support 2.3.0+, pygments changed color names in 2.4.0.
+# startswith is sufficient and avoids potential exceptions from split and int.
+if pygments.__version__.startswith('2.3.'):
+    PYGMENT_COLOR = {
+        'black': '#ansiblack',
+        'red': '#ansidarkred',
+        'green': '#ansidarkgreen',
+        'yellow': '#ansibrown',
+        'blue': '#ansidarkblue',
+        'magenta': '#ansipurple',
+        'cyan': '#ansiteal',
+        'gray': '#ansilightgray',
+        'brightblack': '#ansidarkgray',
+        'brightred': '#ansired',
+        'brightgreen': '#ansigreen',
+        'brightyellow': '#ansiyellow',
+        'brightblue': '#ansiblue',
+        'brightmagenta': '#ansifuchsia',
+        'brightcyan': '#ansiturquoise',
+        'white': '#ansiwhite',
+    }
+else:
+    PYGMENT_COLOR = {
+        'black': 'ansiblack',
+        'red': 'ansired',
+        'green': 'ansigreen',
+        'yellow': 'ansiyellow',
+        'blue': 'ansiblue',
+        'magenta': 'ansimagenta',
+        'cyan': 'ansicyan',
+        'gray': 'ansigray',
+        'brightblack': 'ansibrightblack',
+        'brightred': 'ansibrightred',
+        'brightgreen': 'ansibrightgreen',
+        'brightyellow': 'ansibrightyellow',
+        'brightblue': 'ansibrightblue',
+        'brightmagenta': 'ansibrightmagenta',
+        'brightcyan': 'ansibrightcyan',
+        'white': 'ansiwhite',
+    }
+
+
+def set_env_colors(env_colors=None):
     """
+    Return a dictionary to be used in Pygments custom style class.
+
     Grab custom colors from JC_COLORS environment variable. JC_COLORS env variable takes 4 comma
     separated string values and should be in the format of:
 
@@ -115,40 +165,36 @@ def set_env_colors():
     JC_COLORS=default,default,default,default
 
     """
-    env_colors = os.getenv('JC_COLORS')
     input_error = False
 
     if env_colors:
         color_list = env_colors.split(',')
     else:
+        color_list = ['default', 'default', 'default', 'default']
+
+    if len(color_list) != 4:
         input_error = True
 
-    if env_colors and len(color_list) != 4:
-        print('jc:   Warning: could not parse JC_COLORS environment variable\n', file=sys.stderr)
-        input_error = True
-
-    if env_colors:
-        for color in color_list:
-            if color not in ['black', 'red', 'green', 'yellow', 'blue', 'magenta', 'cyan', 'gray', 'brightblack', 'brightred',
-                             'brightgreen', 'brightyellow', 'brightblue', 'brightmagenta', 'brightcyan', 'white', 'default']:
-                print('jc:   Warning: could not parse JC_COLORS environment variable\n', file=sys.stderr)
-                input_error = True
+    for color in color_list:
+        if color != 'default' and color not in PYGMENT_COLOR:
+            input_error = True
 
     # if there is an issue with the env variable, just set all colors to default and move on
     if input_error:
+        print('jc:   Warning: could not parse JC_COLORS environment variable\n', file=sys.stderr)
         color_list = ['default', 'default', 'default', 'default']
 
     # Try the color set in the JC_COLORS env variable first. If it is set to default, then fall back to default colors
     return {
-        Name.Tag: f'bold ansi{color_list[0]}' if not color_list[0] == 'default' else 'bold ansiblue',   # key names
-        Keyword: f'ansi{color_list[1]}' if not color_list[1] == 'default' else 'ansibrightblack',       # true, false, null
-        Number: f'ansi{color_list[2]}' if not color_list[2] == 'default' else 'ansimagenta',            # numbers
-        String: f'ansi{color_list[3]}' if not color_list[3] == 'default' else 'ansigreen'               # strings
+        Name.Tag: f'bold {PYGMENT_COLOR[color_list[0]]}' if not color_list[0] == 'default' else f"bold {PYGMENT_COLOR['blue']}",   # key names
+        Keyword: PYGMENT_COLOR[color_list[1]] if not color_list[1] == 'default' else PYGMENT_COLOR['brightblack'],                 # true, false, null
+        Number: PYGMENT_COLOR[color_list[2]] if not color_list[2] == 'default' else PYGMENT_COLOR['magenta'],                      # numbers
+        String: PYGMENT_COLOR[color_list[3]] if not color_list[3] == 'default' else PYGMENT_COLOR['green']                         # strings
     }
 
 
 def piped_output():
-    """returns False if stdout is a TTY. True if output is being piped to another program"""
+    """Return False if stdout is a TTY. True if output is being piped to another program"""
     if sys.stdout.isatty():
         return False
     else:
@@ -156,34 +202,34 @@ def piped_output():
 
 
 def ctrlc(signum, frame):
-    """exit with error on SIGINT"""
+    """Exit with error on SIGINT"""
     sys.exit(1)
 
 
 def parser_shortname(parser_argument):
-    """short name of the parser with dashes and no -- prefix"""
+    """Return short name of the parser with dashes and no -- prefix"""
     return parser_argument[2:]
 
 
 def parser_argument(parser):
-    """short name of the parser with dashes and with -- prefix"""
+    """Return short name of the parser with dashes and with -- prefix"""
     return f'--{parser}'
 
 
 def parser_mod_shortname(parser):
-    """short name of the parser's module name (no -- prefix and dashes converted to underscores)"""
+    """Return short name of the parser's module name (no -- prefix and dashes converted to underscores)"""
     return parser.replace('--', '').replace('-', '_')
 
 
 def parser_module(parser):
-    """import the module just in time and return the module object"""
+    """Import the module just in time and return the module object"""
     shortname = parser_mod_shortname(parser)
     path = ('jcparsers.' if shortname in local_parsers else 'jc.parsers.')
     return importlib.import_module(path + shortname)
 
 
 def parsers_text(indent=0, pad=0):
-    """return the argument and description information from each parser"""
+    """Return the argument and description information from each parser"""
     ptext = ''
     for parser in parsers:
         parser_arg = parser_argument(parser)
@@ -201,7 +247,7 @@ def parsers_text(indent=0, pad=0):
 
 
 def about_jc():
-    """return jc info and the contents of each parser.info as a dictionary"""
+    """Return jc info and the contents of each parser.info as a dictionary"""
     parser_list = []
 
     for parser in parsers:
@@ -231,7 +277,7 @@ def about_jc():
 
 
 def helptext(message):
-    """return the help text with the list of parsers"""
+    """Return the help text with the list of parsers"""
     parsers_string = parsers_text(indent=12, pad=17)
 
     helptext_string = f'''
@@ -247,7 +293,7 @@ def helptext(message):
 {parsers_string}
     Options:
             -a               about jc
-            -d               debug - show trace messages
+            -d               debug - show traceback (-dd for verbose traceback)
             -m               monochrome output
             -p               pretty print output
             -q               quiet - suppress warnings
@@ -260,30 +306,30 @@ def helptext(message):
 
             jc -p ls -al
     '''
-    print(textwrap.dedent(helptext_string), file=sys.stderr)
-
-
-def json_out(data, pretty=False, mono=False, piped_out=False):
-    # set colors
-    class JcStyle(Style):
-        styles = set_env_colors()
+    return textwrap.dedent(helptext_string)
 
 
+def json_out(data, pretty=False, env_colors=None, mono=False, piped_out=False):
+    """Return a JSON formatted string. String may include color codes or be pretty printed."""
     if not mono and not piped_out:
+        # set colors
+        class JcStyle(Style):
+            styles = set_env_colors(env_colors)
+
         if pretty:
-            print(highlight(json.dumps(data, indent=2), JsonLexer(), Terminal256Formatter(style=JcStyle))[0:-1])
+            return str(highlight(json.dumps(data, indent=2), JsonLexer(), Terminal256Formatter(style=JcStyle))[0:-1])
         else:
-            print(highlight(json.dumps(data), JsonLexer(), Terminal256Formatter(style=JcStyle))[0:-1])
+            return str(highlight(json.dumps(data), JsonLexer(), Terminal256Formatter(style=JcStyle))[0:-1])
     else:
         if pretty:
-            print(json.dumps(data, indent=2))
+            return json.dumps(data, indent=2)
         else:
-            print(json.dumps(data))
+            return json.dumps(data)
 
 
 def generate_magic_command(args):
     """
-    Returns a tuple with a boolean and a command, where the boolean signifies that
+    Return a tuple with a boolean and a command, where the boolean signifies that
     the command is valid, and the command is either a command string or None.
     """
 
@@ -316,7 +362,7 @@ def generate_magic_command(args):
     magic_dict = {}
     parser_info = about_jc()['parsers']
 
-    # Create a dictionary of magic_commands to their respective parsers.
+    # create a dictionary of magic_commands to their respective parsers.
     for entry in parser_info:
         # Update the dict with all of the magic commands for this parser, if they exist.
         magic_dict.update({mc: entry['argument'] for mc in entry.get('magic_commands', [])})
@@ -325,7 +371,7 @@ def generate_magic_command(args):
     one_word_command = args_given[0]
     two_word_command = ' '.join(args_given[0:2])
 
-    # Try to get a parser for two_word_command, otherwise get one for one_word_command
+    # try to get a parser for two_word_command, otherwise get one for one_word_command
     found_parser = magic_dict.get(two_word_command, magic_dict.get(one_word_command))
 
     # construct a new command line using the standard syntax: COMMAND | jc --PARSER -OPTIONS
@@ -338,6 +384,7 @@ def generate_magic_command(args):
 
 
 def magic():
+    """Runs the command generated by generate_magic_command() to support magic syntax"""
     valid_command, run_command = generate_magic_command(sys.argv)
     if valid_command:
         os.system(run_command)
@@ -345,7 +392,7 @@ def magic():
     elif run_command is None:
         return
     else:
-        helptext(f'parser not found for "{run_command}"')
+        print(helptext(f'parser not found for "{run_command}"'), file=sys.stderr)
         sys.exit(1)
 
 
@@ -359,6 +406,8 @@ def main():
     except AttributeError:
         pass
 
+    jc_colors = os.getenv('JC_COLORS')
+
     # try magic syntax first: e.g. jc -p ls -al
     magic()
 
@@ -370,54 +419,54 @@ def main():
             options.extend(opt[1:])
 
     debug = 'd' in options
+    verbose_debug = True if options.count('d') > 1 else False
     mono = 'm' in options
     pretty = 'p' in options
     quiet = 'q' in options
     raw = 'r' in options
 
+    if verbose_debug:
+        import jc.tracebackplus
+        jc.tracebackplus.enable(context=11)
+
     if 'a' in options:
-        json_out(about_jc(), pretty=pretty, mono=mono, piped_out=piped_output())
+        print(json_out(about_jc(), pretty=pretty, env_colors=jc_colors, mono=mono, piped_out=piped_output()))
         sys.exit(0)
 
     if sys.stdin.isatty():
-        helptext('missing piped data')
+        print(helptext('missing piped data'), file=sys.stderr)
         sys.exit(1)
 
     data = sys.stdin.read()
 
     found = False
 
-    if debug:
-        for arg in sys.argv:
-            parser_name = parser_shortname(arg)
+    for arg in sys.argv:
+        parser_name = parser_shortname(arg)
 
-            if parser_name in parsers:
-                # load parser module just in time so we don't need to load all modules
-                parser = parser_module(arg)
+        if parser_name in parsers:
+            # load parser module just in time so we don't need to load all modules
+            parser = parser_module(arg)
+            try:
                 result = parser.parse(data, raw=raw, quiet=quiet)
                 found = True
                 break
-    else:
-        for arg in sys.argv:
-            parser_name = parser_shortname(arg)
 
-            if parser_name in parsers:
-                # load parser module just in time so we don't need to load all modules
-                parser = parser_module(arg)
-                try:
-                    result = parser.parse(data, raw=raw, quiet=quiet)
-                    found = True
-                    break
-                except Exception:
+            except Exception:
+                if debug:
+                    raise
+                else:
+                    import jc.utils
                     jc.utils.error_message(
-                        f'{parser_name} parser could not parse the input data. Did you use the correct parser?\n         For details use the -d option.')
+                        f'{parser_name} parser could not parse the input data. Did you use the correct parser?\n'
+                        '                 For details use the -d or -dd option.')
                     sys.exit(1)
 
     if not found:
-        helptext('missing or incorrect arguments')
+        print(helptext('missing or incorrect arguments'), file=sys.stderr)
         sys.exit(1)
 
-    json_out(result, pretty=pretty, mono=mono, piped_out=piped_output())
+    print(json_out(result, pretty=pretty, env_colors=jc_colors, mono=mono, piped_out=piped_output()))
 
 
 if __name__ == '__main__':
