@@ -61,7 +61,7 @@ import jc.utils
 
 
 class info():
-    version = '1.1'
+    version = '1.2'
     description = '/etc/hosts file parser'
     author = 'Kelly Brazil'
     author_email = 'kellyjonbrazil@gmail.com'
@@ -117,12 +117,12 @@ def parse(data, raw=False, quiet=False):
         jc.utils.compatibility(__name__, info.compatible)
 
     raw_output = []
-    cleandata = data.splitlines()
 
     # Clear any blank lines
-    cleandata = list(filter(None, cleandata))
+    cleandata = list(filter(None, data.splitlines()))
+
+    if jc.utils.has_data(data):
 
-    if cleandata:
         for line in cleandata:
             output_line = {}
             # ignore commented lines
