@@ -134,7 +134,7 @@ import jc.utils
 
 
 class info():
-    version = '1.2'
+    version = '1.4'
     description = 'iptables command parser'
     author = 'Kelly Brazil'
     author_email = 'kellyjonbrazil@gmail.com'
@@ -194,19 +194,19 @@ def process(proc_data):
             if 'bytes' in rule:
                 multiplier = 1
                 if rule['bytes'][-1] == 'K':
-                    multiplier = 1000
+                    multiplier = 10 ** 3
                     rule['bytes'] = rule['bytes'].rstrip('K')
                 elif rule['bytes'][-1] == 'M':
-                    multiplier = 1000000
+                    multiplier = 10 ** 6
                     rule['bytes'] = rule['bytes'].rstrip('M')
                 elif rule['bytes'][-1] == 'G':
-                    multiplier = 1000000000
+                    multiplier = 10 ** 9
                     rule['bytes'] = rule['bytes'].rstrip('G')
                 elif rule['bytes'][-1] == 'T':
-                    multiplier = 1000000000000
+                    multiplier = 10 ** 12
                     rule['bytes'] = rule['bytes'].rstrip('T')
                 elif rule['bytes'][-1] == 'P':
-                    multiplier = 1000000000000000
+                    multiplier = 10 ** 15
                     rule['bytes'] = rule['bytes'].rstrip('P')
 
                 try:
@@ -243,36 +243,39 @@ def parse(data, raw=False, quiet=False):
     chain = {}
     headers = []
 
-    cleandata = data.splitlines()
+    if jc.utils.has_data(data):
 
-    for line in cleandata:
+        for line in list(filter(None, data.splitlines())):
 
-        if line.startswith('Chain'):
+            if line.startswith('Chain'):
+                if chain:
+                    raw_output.append(chain)
+
+                chain = {}
+                headers = []
+
+                parsed_line = line.split()
+
+                chain['chain'] = parsed_line[1]
+                chain['rules'] = []
+
+                continue
+
+            elif line.startswith('target') or line.find('pkts') == 1 or line.startswith('num'):
+                headers = []
+                headers = [h for h in ' '.join(line.lower().strip().split()).split() if h]
+                headers.append("options")
+
+                continue
+
+            else:
+                rule = line.split(maxsplit=len(headers) - 1)
+                temp_rule = dict(zip(headers, rule))
+                if temp_rule:
+                    chain['rules'].append(temp_rule)
+
+        if chain:
             raw_output.append(chain)
-            chain = {}
-            headers = []
-
-            parsed_line = line.split()
-
-            chain['chain'] = parsed_line[1]
-            chain['rules'] = []
-
-            continue
-
-        elif line.startswith('target') or line.find('pkts') == 1 or line.startswith('num'):
-            headers = []
-            headers = [h for h in ' '.join(line.lower().strip().split()).split() if h]
-            headers.append("options")
-
-            continue
-
-        else:
-            rule = line.split(maxsplit=len(headers) - 1)
-            temp_rule = dict(zip(headers, rule))
-            if temp_rule:
-                chain['rules'].append(temp_rule)
-
-    raw_output = list(filter(None, raw_output))
 
     if raw:
         return raw_output
